# -*- coding: utf-8 -*-#
# -------------------------------------------------------------------------------
# Name:         utils.py
# Description:  
# Author:       xaoyaoo
# Date:         2024/01/16
# -------------------------------------------------------------------------------
import base64
import json
import logging
import os
import re
import traceback
from .rjson import ReJson
from functools import wraps


def read_session_local_wxid(session_file):
    try:
        with open(session_file, 'r') as f:
            session = json.load(f)
    except FileNotFoundError:
        logging.error(f"Session file not found: {session_file}")
        return None
    except json.JSONDecodeError as e:
        logging.error(f"Error decoding JSON file: {e}")
        return None
    rdata = [k for k in session.keys() if k != "test"]
    return rdata


def read_session(session_file, wxid, arg):
    try:
        with open(session_file, 'r') as f:
            session = json.load(f)
    except FileNotFoundError:
        logging.error(f"Session file not found: {session_file}")
        return None
    except json.JSONDecodeError as e:
        logging.error(f"Error decoding JSON file: {e}")
        return None
    return session.get(wxid, {}).get(arg, None)


def get_session_wxids(session_file):
    try:
        with open(session_file, 'r') as f:
            session = json.load(f)
    except FileNotFoundError:
        logging.error(f"Session file not found: {session_file}")
        return None
    except json.JSONDecodeError as e:
        logging.error(f"Error decoding JSON file: {e}")
        return None
    return list(session.keys())


def save_session(session_file, wxid, arg, value):
    try:
        with open(session_file, 'r') as f:
            session = json.load(f)
    except FileNotFoundError:
        session = {}
    except json.JSONDecodeError as e:
        logging.error(f"Error decoding JSON file: {e}")
        return False

    if wxid not in session:
        session[wxid] = {}
    if not isinstance(session[wxid], dict):
        session[wxid] = {}
    session[wxid][arg] = value
    try:
        with open(session_file, 'w') as f:
            json.dump(session, f, indent=4, ensure_ascii=False)
    except Exception as e:
        logging.error(f"Error writing to file: {e}")
        return False
    return True


def validate_title(title):
    """
    校验文件名是否合法
    """
    rstr = r"[\/\\\:\*\?\"\<\>\|\.]"  # '/ \ : * ? " < > |'
    new_title = re.sub(rstr, "_", title)  # 替换为下划线
    return new_title


def error9999(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            traceback_data = traceback.format_exc()
            rdata = f"{traceback_data}"
            # logging.error(rdata)
            return ReJson(9999, body=rdata)

    return wrapper


def gen_base64(path):
    # 获取文件名后缀
    extension = os.path.splitext(path)[1]
    if extension == '.js':
        start_str = 'data:text/javascript;base64,'
    elif extension == '.css':
        start_str = 'data:text/css;base64,'
    elif extension == '.html':
        start_str = 'data:text/html;base64,'
    elif extension == '.json':
        start_str = 'data:application/json;base64,'
    else:
        start_str = 'data:text/plain;base64,'

    with open(path, 'rb') as file:
        js_code = file.read()

    base64_encoded_js = base64.b64encode(js_code).decode('utf-8')
    return start_str + base64_encoded_js



# -*- coding: utf-8 -*-#
# -------------------------------------------------------------------------------
# Name:         chat_api.py
# Description:  
# Author:       xaoyaoo
# Date:         2024/01/02
# -------------------------------------------------------------------------------
import base64
import json
import logging
import os
import re
import time
import shutil
import pythoncom
import pywxdump

from flask import Flask, request, render_template, g, Blueprint, send_file, make_response, session
from pywxdump import get_core_db, all_merge_real_time_db
from pywxdump.api.rjson import ReJson, RqJson
from pywxdump.api.utils import read_session, get_session_wxids, save_session, error9999, gen_base64, validate_title, \
    read_session_local_wxid
from pywxdump import read_info, VERSION_LIST, batch_decrypt, BiasAddr, merge_db, decrypt_merge, merge_real_time_db

from pywxdump.dbpreprocess import wxid2userinfo, ParsingMSG, get_user_list, get_recent_user_list, ParsingMediaMSG, \
    download_file, export_csv, export_json, ParsingMicroMsg
from pywxdump.dbpreprocess.utils import dat2img

# app = Flask(__name__, static_folder='../ui/web/dist', static_url_path='/')

api = Blueprint('api', __name__, template_folder='../ui/web', static_folder='../ui/web/assets/', )
api.debug = False


# 以下为初始化相关 *******************************************************************************************************
@api.route('/api/init_last_local_wxid', methods=["GET", 'POST'])
@error9999
def init_last_local_wxid():
    """
    初始化，包括key
    :return:
    """
    local_wxid = read_session_local_wxid(g.sf)
    if local_wxid:
        return ReJson(0, {"local_wxids": local_wxid})
    return ReJson(0, {"local_wxids": []})


@api.route('/api/init_last', methods=["GET", 'POST'])
@error9999
def init_last():
    """
    是否初始化
    :return:
    """
    my_wxid = request.json.get("my_wxid", "")
    my_wxid = my_wxid.strip().strip("'").strip('"') if isinstance(my_wxid, str) else ""
    if not my_wxid:
        my_wxid = read_session(g.sf, "test", "last")
    if my_wxid:
        save_session(g.sf, "test", "last", my_wxid)
        merge_path = read_session(g.sf, my_wxid, "merge_path")
        wx_path = read_session(g.sf, my_wxid, "wx_path")
        key = read_session(g.sf, my_wxid, "key")
        rdata = {
            "merge_path": merge_path,
            "wx_path": wx_path,
            "key": key,
            "my_wxid": my_wxid,
            "is_init": True,
        }
        if merge_path and wx_path:
            return ReJson(0, rdata)
    return ReJson(0, {"is_init": False, "my_wxid": ""})


@api.route('/api/init_key', methods=["GET", 'POST'])
@error9999
def init_key():
    """
    初始化，包括key
    :return:
    """
    wx_path = request.json.get("wx_path", "").strip().strip("'").strip('"')
    key = request.json.get("key", "").strip().strip("'").strip('"')
    my_wxid = request.json.get("my_wxid", "").strip().strip("'").strip('"')
    if not wx_path:
        return ReJson(1002, body=f"wx_path is required: {wx_path}")
    if not os.path.exists(wx_path):
        return ReJson(1001, body=f"wx_path not exists: {wx_path}")
    if not key:
        return ReJson(1002, body=f"key is required: {key}")
    if not my_wxid:
        return ReJson(1002, body=f"my_wxid is required: {my_wxid}")

    old_merge_save_path = read_session(g.sf, my_wxid, "merge_path")
    if isinstance(old_merge_save_path, str) and old_merge_save_path and os.path.exists(old_merge_save_path):
        pmsg = ParsingMSG(old_merge_save_path)
        pmsg.close_all_connection()

    out_path = os.path.join(g.tmp_path, "decrypted", my_wxid) if my_wxid else os.path.join(g.tmp_path, "decrypted")
    # 检查文件夹中文件是否被占用
    if os.path.exists(out_path):
        try:
            shutil.rmtree(out_path)
        except PermissionError as e:
            # 显示堆栈信息
            logging.error(f"{e}", exc_info=True)
            return ReJson(2001, body=str(e))

    code, merge_save_path = decrypt_merge(wx_path=wx_path, key=key, outpath=out_path)
    time.sleep(1)
    if code:
        # 移动merge_save_path到g.tmp_path/my_wxid
        if not os.path.exists(os.path.join(g.tmp_path, my_wxid)):
            os.makedirs(os.path.join(g.tmp_path, my_wxid))
        merge_save_path_new = os.path.join(g.tmp_path, my_wxid, "merge_all.db")
        shutil.move(merge_save_path, str(merge_save_path_new))

        # 删除out_path
        if os.path.exists(out_path):
            try:
                shutil.rmtree(out_path)
            except PermissionError as e:
                # 显示堆栈信息
                logging.error(f"{e}", exc_info=True)

        save_session(g.sf, my_wxid, "merge_path", merge_save_path_new)
        save_session(g.sf, my_wxid, "wx_path", wx_path)
        save_session(g.sf, my_wxid, "key", key)
        save_session(g.sf, my_wxid, "my_wxid", my_wxid)
        save_session(g.sf, "test", "last", my_wxid)
        rdata = {
            "merge_path": merge_save_path,
            "wx_path": wx_path,
            "key": key,
            "my_wxid": my_wxid,
            "is_init": True,
        }
        return ReJson(0, rdata)
    else:
        return ReJson(2001, body=merge_save_path)


@api.route('/api/init_nokey', methods=["GET", 'POST'])
@error9999
def init_nokey():
    """
    初始化，包括key
    :return:
    """
    merge_path = request.json.get("merge_path", "").strip().strip("'").strip('"')
    wx_path = request.json.get("wx_path", "").strip().strip("'").strip('"')
    my_wxid = request.json.get("my_wxid", "").strip().strip("'").strip('"')

    if not wx_path:
        return ReJson(1002, body=f"wx_path is required: {wx_path}")
    if not os.path.exists(wx_path):
        return ReJson(1001, body=f"wx_path not exists: {wx_path}")
    if not merge_path:
        return ReJson(1002, body=f"merge_path is required: {merge_path}")
    if not my_wxid:
        return ReJson(1002, body=f"my_wxid is required: {my_wxid}")

    key = read_session(g.sf, my_wxid, "key")

    save_session(g.sf, my_wxid, "merge_path", merge_path)
    save_session(g.sf, my_wxid, "wx_path", wx_path)
    save_session(g.sf, my_wxid, "key", key)
    save_session(g.sf, my_wxid, "my_wxid", my_wxid)
    save_session(g.sf, "test", "last", my_wxid)
    rdata = {
        "merge_path": merge_path,
        "wx_path": wx_path,
        "key": "",
        "my_wxid": my_wxid,
        "is_init": True,
    }
    return ReJson(0, rdata)


# END 以上为初始化相关 ***************************************************************************************************


# start 以下为聊天联系人相关api *******************************************************************************************

@api.route('/api/recent_user_list', methods=["GET", 'POST'])
@error9999
def recent_user_list():
    """
    获取联系人列表
    :return:
    """
    my_wxid = read_session(g.sf, "test", "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")
    merge_path = read_session(g.sf, my_wxid, "merge_path")
    user_list = get_recent_user_list(merge_path, merge_path, limit=200)
    return ReJson(0, user_list)


@api.route('/api/user_labels_dict', methods=["GET", 'POST'])
@error9999
def user_labels_dict():
    """
    获取标签字典
    :return:
    """
    my_wxid = read_session(g.sf, "test", "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")
    merge_path = read_session(g.sf, my_wxid, "merge_path")
    user_labels_dict = ParsingMicroMsg(merge_path).labels_dict()
    return ReJson(0, user_labels_dict)


@api.route('/api/user_list', methods=["GET", 'POST'])
@error9999
def user_list():
    """
    获取联系人列表
    :return:
    """
    if request.method == "GET":
        word = request.args.get("word", "")
    elif request.method == "POST":
        word = request.json.get("word", "")
    else:
        return ReJson(1003, msg="Unsupported method")
    my_wxid = read_session(g.sf, "test", "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")
    merge_path = read_session(g.sf, my_wxid, "merge_path")
    user_list = get_user_list(merge_path, merge_path, word)
    return ReJson(0, user_list)


@api.route('/api/wxid2user', methods=["GET", 'POST'])
@error9999
def wxid2user():
    """
    获取联系人列表
    :return:
    """
    if request.method == "GET":
        word = request.args.get("wxid", "")
    elif request.method == "POST":
        word = request.json.get("wxid", "")
    else:
        return ReJson(1003, msg="Unsupported method")

    my_wxid = read_session(g.sf, "test", "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")
    merge_path = read_session(g.sf, my_wxid, "merge_path")
    user_info = wxid2userinfo(merge_path, merge_path, wxid=word)
    return ReJson(0, user_info)


@api.route('/api/mywxid', methods=["GET", 'POST'])
@error9999
def mywxid():
    """
    获取我的微信id
    :return:
    """
    my_wxid = read_session(g.sf, "test", "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")
    return ReJson(0, {"my_wxid": my_wxid})


# end 以上为聊天联系人相关api *********************************************************************************************

# start 以下为聊天记录相关api *********************************************************************************************

@api.route('/api/realtimemsg', methods=["GET", "POST"])
@error9999
def get_real_time_msg():
    """
    获取实时消息 使用 merge_real_time_db()函数
    :return:
    """
    my_wxid = read_session(g.sf, "test", "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")

    merge_path = read_session(g.sf, my_wxid, "merge_path")
    key = read_session(g.sf, my_wxid, "key")
    wx_path = read_session(g.sf, my_wxid, "wx_path")

    if not merge_path or not key or not wx_path or not wx_path:
        return ReJson(1002, body="msg_path or media_path or wx_path or key is required")

    code, ret = all_merge_real_time_db(key=key, wx_path=wx_path, merge_path=merge_path)
    if code:
        return ReJson(0, ret)
    else:
        return ReJson(2001, body=ret)


@api.route('/api/msg_count', methods=["GET", 'POST'])
@error9999
def msg_count():
    """
    获取联系人的聊天记录数量
    :return:
    """
    if request.method == "GET":
        wxid = request.args.get("wxid")
    elif request.method == "POST":
        wxid = request.json.get("wxid")
    else:
        return ReJson(1003, msg="Unsupported method")

    my_wxid = read_session(g.sf, "test", "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")
    merge_path = read_session(g.sf, my_wxid, "merge_path")
    chat_count = ParsingMSG(merge_path).msg_count(wxid)
    return ReJson(0, chat_count)


@api.route('/api/imgsrc/<path:imgsrc>', methods=["GET", 'POST'])
def get_imgsrc(imgsrc):
    """
    获取图片,从网络获取图片，主要功能只是下载图片，缓存到本地
    :return:
    """
    if not imgsrc:
        return ReJson(1002)

    if imgsrc.startswith("FileStorage"):  # 如果是本地图片文件则调用get_img
        return get_img(imgsrc)

    # 将?后面的参数连接到imgsrc
    imgsrc = imgsrc + "?" + request.query_string.decode("utf-8") if request.query_string else imgsrc

    my_wxid = read_session(g.sf, "test", "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")

    img_tmp_path = os.path.join(g.tmp_path, my_wxid, "imgsrc")
    if not os.path.exists(img_tmp_path):
        os.makedirs(img_tmp_path)
    file_name = imgsrc.replace("http://", "").replace("https://", "").replace("/", "_").replace("?", "_")
    file_name = file_name + ".jpg"
    # 如果文件名过长，则将文件明分为目录和文件名
    if len(file_name) > 255:
        file_name = file_name[:255] + "/" + file_name[255:]

    img_path_all = os.path.join(img_tmp_path, file_name)
    if os.path.exists(img_path_all):
        return send_file(img_path_all)
    else:
        download_file(imgsrc, img_path_all)
        if os.path.exists(img_path_all):
            return send_file(img_path_all)
        else:
            return ReJson(4004, body=imgsrc)


@api.route('/api/img/<path:img_path>', methods=["GET", 'POST'])
@error9999
def get_img(img_path):
    """
    获取图片
    :return:
    """

    if not img_path:
        return ReJson(1002)

    my_wxid = read_session(g.sf, "test", "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")
    wx_path = read_session(g.sf, my_wxid, "wx_path")

    img_path = img_path.replace("\\\\", "\\")

    img_tmp_path = os.path.join(g.tmp_path, my_wxid, "img")
    original_img_path = os.path.join(wx_path, img_path)
    if os.path.exists(original_img_path):
        fomt, md5, out_bytes = dat2img(original_img_path)
        imgsavepath = os.path.join(img_tmp_path, img_path + "_" + ".".join([md5, fomt]))
        if not os.path.exists(os.path.dirname(imgsavepath)):
            os.makedirs(os.path.dirname(imgsavepath))
        with open(imgsavepath, "wb") as f:
            f.write(out_bytes)
        return send_file(imgsavepath)
    else:
        return ReJson(1001, body=original_img_path)


@api.route('/api/msgs', methods=["GET", 'POST'])
@error9999
def get_msgs():
    my_wxid = read_session(g.sf, "test", "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")
    merge_path = read_session(g.sf, my_wxid, "merge_path")

    start = request.json.get("start")
    limit = request.json.get("limit")
    wxid = request.json.get("wxid")

    if not wxid:
        return ReJson(1002, body=f"wxid is required: {wxid}")
    if start and isinstance(start, str) and start.isdigit():
        start = int(start)
    if limit and isinstance(limit, str) and limit.isdigit():
        limit = int(limit)
    if start is None or limit is None:
        return ReJson(1002, body=f"start or limit is required {start} {limit}")
    if not isinstance(start, int) and not isinstance(limit, int):
        return ReJson(1002, body=f"start or limit is not int {start} {limit}")

    parsing_msg = ParsingMSG(merge_path)
    msgs, wxid_list = parsing_msg.msg_list(wxid, start, limit)
    wxid_list.append(my_wxid)
    user_list = wxid2userinfo(merge_path, merge_path, wxid_list)
    return ReJson(0, {"msg_list": msgs, "user_list": user_list})


@api.route('/api/video/<path:videoPath>', methods=["GET", 'POST'])
def get_video(videoPath):
    my_wxid = read_session(g.sf, "test", "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")
    wx_path = read_session(g.sf, my_wxid, "wx_path")

    videoPath = videoPath.replace("\\\\", "\\")

    video_tmp_path = os.path.join(g.tmp_path, my_wxid, "video")
    original_img_path = os.path.join(wx_path, videoPath)
    if not os.path.exists(original_img_path):
        return ReJson(5002)
    # 复制文件到临时文件夹
    video_save_path = os.path.join(video_tmp_path, videoPath)
    if not os.path.exists(os.path.dirname(video_save_path)):
        os.makedirs(os.path.dirname(video_save_path))
    if os.path.exists(video_save_path):
        return send_file(video_save_path)
    shutil.copy(original_img_path, video_save_path)
    return send_file(original_img_path)


@api.route('/api/audio/<path:savePath>', methods=["GET", 'POST'])
def get_audio(savePath):
    my_wxid = read_session(g.sf, "test", "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")
    merge_path = read_session(g.sf, my_wxid, "merge_path")

    savePath = os.path.join(g.tmp_path, my_wxid, "audio", savePath)  # 这个是从url中获取的
    if os.path.exists(savePath):
        return send_file(savePath)

    MsgSvrID = savePath.split("_")[-1].replace(".wav", "")
    if not savePath:
        return ReJson(1002)

    # 判断savePath路径的文件夹是否存在
    if not os.path.exists(os.path.dirname(savePath)):
        os.makedirs(os.path.dirname(savePath))

    parsing_media_msg = ParsingMediaMSG(merge_path)
    wave_data = parsing_media_msg.get_audio(MsgSvrID, is_play=False, is_wave=True, save_path=savePath, rate=24000)
    if not wave_data:
        return ReJson(1001, body="wave_data is required")

    if os.path.exists(savePath):
        return send_file(savePath)
    else:
        return ReJson(4004, body=savePath)


@api.route('/api/file_info', methods=["GET", 'POST'])
def get_file_info():
    file_path = request.args.get("file_path")
    file_path = request.json.get("file_path", file_path)
    if not file_path:
        return ReJson(1002)

    my_wxid = read_session(g.sf, "test", "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")
    wx_path = read_session(g.sf, my_wxid, "wx_path")

    all_file_path = os.path.join(wx_path, file_path)
    if not os.path.exists(all_file_path):
        return ReJson(5002)
    file_name = os.path.basename(all_file_path)
    file_size = os.path.getsize(all_file_path)
    return ReJson(0, {"file_name": file_name, "file_size": str(file_size)})


@api.route('/api/file/<path:filePath>', methods=["GET", 'POST'])
def get_file(filePath):
    my_wxid = read_session(g.sf, "test", "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")
    wx_path = read_session(g.sf, my_wxid, "wx_path")

    all_file_path = os.path.join(wx_path, filePath)
    if not os.path.exists(all_file_path):
        return ReJson(5002)
    return send_file(all_file_path)


# end 以上为聊天记录相关api *********************************************************************************************

# start 导出聊天记录 *****************************************************************************************************

@api.route('/api/export_endb', methods=["GET", 'POST'])
def get_export_endb():
    """
    导出加密数据库
    :return:
    """
    my_wxid = read_session(g.sf, "test", "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")
    wx_path = read_session(g.sf, my_wxid, "wx_path")
    wx_path = request.json.get("wx_path", wx_path)

    if not wx_path:
        return ReJson(1002, body=f"wx_path is required: {wx_path}")
    if not os.path.exists(wx_path):
        return ReJson(1001, body=f"wx_path not exists: {wx_path}")

    # 分割wx_path的文件名和父目录
    code, wxdbpaths = get_core_db(wx_path)
    if not code:
        return ReJson(2001, body=wxdbpaths)

    outpath = os.path.join(g.tmp_path, "export", my_wxid, "endb")
    if not os.path.exists(outpath):
        os.makedirs(outpath)

    for wxdb in wxdbpaths:
        # 复制wxdb->outpath, os.path.basename(wxdb)
        assert isinstance(outpath, str)  # 为了解决pycharm的警告, 无实际意义
        shutil.copy(wxdb, os.path.join(outpath, os.path.basename(wxdb)))
    return ReJson(0, body=outpath)


@api.route('/api/export_dedb', methods=["GET", "POST"])
def get_export_dedb():
    """
    导出解密数据库
    :return:
    """
    my_wxid = read_session(g.sf, "test", "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")

    key = request.json.get("key", read_session(g.sf, my_wxid, "key"))
    wx_path = request.json.get("wx_path", read_session(g.sf, my_wxid, "wx_path"))

    if not key:
        return ReJson(1002, body=f"key is required: {key}")
    if not wx_path:
        return ReJson(1002, body=f"wx_path is required: {wx_path}")
    if not os.path.exists(wx_path):
        return ReJson(1001, body=f"wx_path not exists: {wx_path}")

    outpath = os.path.join(g.tmp_path, "export", my_wxid, "dedb")
    if not os.path.exists(outpath):
        os.makedirs(outpath)

    code, merge_save_path = decrypt_merge(wx_path=wx_path, key=key, outpath=outpath)
    time.sleep(1)
    if code:
        return ReJson(0, body=merge_save_path)
    else:
        return ReJson(2001, body=merge_save_path)


@api.route('/api/export_csv', methods=["GET", 'POST'])
def get_export_csv():
    """
    导出csv
    :return:
    """
    my_wxid = read_session(g.sf, "test", "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")

    wxid = request.json.get("wxid")
    # st_ed_time = request.json.get("datetime", [0, 0])
    if not wxid:
        return ReJson(1002, body=f"username is required: {wxid}")
    # if not isinstance(st_ed_time, list) or len(st_ed_time) != 2:
    #     return ReJson(1002, body=f"datetime is required: {st_ed_time}")
    # start, end = st_ed_time
    # if not isinstance(start, int) or not isinstance(end, int) or start >= end:
    #     return ReJson(1002, body=f"datetime is required: {st_ed_time}")

    outpath = os.path.join(g.tmp_path, "export", my_wxid, "csv", wxid)
    if not os.path.exists(outpath):
        os.makedirs(outpath)

    code, ret = export_csv(wxid, outpath, read_session(g.sf, my_wxid, "merge_path"))
    if code:
        return ReJson(0, ret)
    else:
        return ReJson(2001, body=ret)


@api.route('/api/export_json', methods=["GET", 'POST'])
def get_export_json():
    """
    导出json
    :return:
    """
    my_wxid = read_session(g.sf, "test", "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")

    wxid = request.json.get("wxid")
    if not wxid:
        return ReJson(1002, body=f"username is required: {wxid}")

    outpath = os.path.join(g.tmp_path, "export", my_wxid, "json", wxid)
    if not os.path.exists(outpath):
        os.makedirs(outpath)

    code, ret = export_json(wxid, outpath, read_session(g.sf, my_wxid, "merge_path"))
    if code:
        return ReJson(0, ret)
    else:
        return ReJson(2001, body=ret)


# @api.route('/api/export', methods=["GET", 'POST'])
# @error9999
# def export():
#     """
#     导出聊天记录
#     :return:
#     """
#     export_type = request.json.get("export_type")
#     start_time = request.json.get("start_time", 0)
#     end_time = request.json.get("end_time", 0)
#     chat_type = request.json.get("chat_type")
#     username = request.json.get("username")
#
#     wx_path = request.json.get("wx_path", read_session(g.sf, "wx_path"))
#     key = request.json.get("key", read_session(g.sf, "key"))
#
#     if not export_type or not isinstance(export_type, str):
#         return ReJson(1002)
#
#     # 导出路径
#     outpath = os.path.join(g.tmp_path, "export", export_type)
#     if not os.path.exists(outpath):
#         os.makedirs(outpath)
#
#     if export_type == "endb":  # 导出加密数据库
#         # 获取微信文件夹路径
#         if not wx_path:
#             return ReJson(1002)
#         if not os.path.exists(wx_path):
#             return ReJson(1001, body=wx_path)
#
#         # 分割wx_path的文件名和父目录
#         code, wxdbpaths = get_core_db(wx_path)
#         if not code:
#             return ReJson(2001, body=wxdbpaths)
#
#         for wxdb in wxdbpaths:
#             # 复制wxdb->outpath, os.path.basename(wxdb)
#             shutil.copy(wxdb, os.path.join(outpath, os.path.basename(wxdb)))
#         return ReJson(0, body=outpath)
#
#     elif export_type == "dedb":
#         if isinstance(start_time, int) and isinstance(end_time, int):
#             msg_path = read_session(g.sf, "msg_path")
#             micro_path = read_session(g.sf, "micro_path")
#             media_path = read_session(g.sf, "media_path")
#             dbpaths = [msg_path, media_path, micro_path]
#             dbpaths = list(set(dbpaths))
#             mergepath = merge_db(dbpaths, os.path.join(outpath, "merge.db"), start_time, end_time)
#             return ReJson(0, body=mergepath)
#             # if msg_path == media_path and msg_path == media_path:
#             #     shutil.copy(msg_path, os.path.join(outpath, "merge.db"))
#             #     return ReJson(0, body=msg_path)
#             # else:
#             #     dbpaths = [msg_path, msg_path, micro_path]
#             #     dbpaths = list(set(dbpaths))
#             #     mergepath = merge_db(dbpaths, os.path.join(outpath, "merge.db"), start_time,  end_time)
#             #     return ReJson(0, body=mergepath)
#         else:
#             return ReJson(1002, body={"start_time": start_time, "end_time": end_time})
#
#     elif export_type == "csv":
#         outpath = os.path.join(outpath, username)
#         if not os.path.exists(outpath):
#             os.makedirs(outpath)
#         code, ret = analyzer.export_csv(username, outpath, read_session(g.sf, "msg_path"))
#         if code:
#             return ReJson(0, ret)
#         else:
#             return ReJson(2001, body=ret)
#     elif export_type == "json":
#         outpath = os.path.join(outpath, username)
#         if not os.path.exists(outpath):
#             os.makedirs(outpath)
#         code, ret = analyzer.export_json(username, outpath, read_session(g.sf, "msg_path"))
#         if code:
#             return ReJson(0, ret)
#         else:
#             return ReJson(2001, body=ret)
#     elif export_type == "html":
#         outpath = os.path.join(outpath, username)
#         if os.path.exists(outpath):
#             shutil.rmtree(outpath)
#         if not os.path.exists(outpath):
#             os.makedirs(outpath)
#         # chat_type_tups = []
#         # for ct in chat_type:
#         #     tup = analyzer.get_name_typeid(ct)
#         #     if tup:
#         #         chat_type_tups += tup
#         # if not chat_type_tups:
#         #     return ReJson(1002)
#
#         # 复制文件 html
#         export_html = os.path.join(os.path.dirname(pywxdump.VERSION_LIST_PATH), "ui", "export")
#         indexhtml_path = os.path.join(export_html, "index.html")
#         assets_path = os.path.join(export_html, "assets")
#         if not os.path.exists(indexhtml_path) or not os.path.exists(assets_path):
#             return ReJson(1001)
#         js_path = ""
#         css_path = ""
#         for file in os.listdir(assets_path):
#             if file.endswith('.js'):
#                 js_path = os.path.join(assets_path, file)
#             elif file.endswith('.css'):
#                 css_path = os.path.join(assets_path, file)
#             else:
#                 continue
#         # 读取html,js,css
#         with open(indexhtml_path, 'r', encoding='utf-8') as f:
#             html = f.read()
#         with open(js_path, 'r', encoding='utf-8') as f:
#             js = f.read()
#         with open(css_path, 'r', encoding='utf-8') as f:
#             css = f.read()
#
#         html = re.sub(r'<script .*?></script>', '', html)  # 删除所有的script标签
#         html = re.sub(r'<link rel="stylesheet" .*?>', '', html)  # 删除所有的link标签
#
#         html = html.replace('</head>', f'<style>{css}</style></head>')
#         html = html.replace('</head>', f'<script type="module" crossorigin>{js}</script></head>')
#         # END 生成index.html
#
#         rdata = func_get_msgs(0, 10000000, username, "", "")
#
#         msg_list = rdata["msg_list"]
#         for i in range(len(msg_list)):
#             if msg_list[i]["type_name"] == "语音":
#                 savePath = msg_list[i]["content"]["src"]
#                 MsgSvrID = savePath.split("_")[-1].replace(".wav", "")
#                 if not savePath:
#                     continue
#                 media_path = read_session(g.sf, "media_path")
#                 wave_data = read_audio(MsgSvrID, is_wave=True, DB_PATH=media_path)
#                 if not wave_data:
#                     continue
#                 # 判断savePath路径的文件夹是否存在
#                 savePath = os.path.join(outpath, savePath)
#                 if not os.path.exists(os.path.dirname(savePath)):
#                     os.makedirs(os.path.dirname(savePath))
#                 with open(savePath, "wb") as f:
#                     f.write(wave_data)
#             elif msg_list[i]["type_name"] == "图片":
#                 img_path = msg_list[i]["content"]["src"]
#                 wx_path = read_session(g.sf, "wx_path")
#                 img_path_all = os.path.join(wx_path, img_path)
#
#                 if os.path.exists(img_path_all):
#                     fomt, md5, out_bytes = read_img_dat(img_path_all)
#                     imgsavepath = os.path.join(outpath, "img", img_path + "_" + ".".join([md5, fomt]))
#                     if not os.path.exists(os.path.dirname(imgsavepath)):
#                         os.makedirs(os.path.dirname(imgsavepath))
#                     with open(imgsavepath, "wb") as f:
#                         f.write(out_bytes)
#                     msg_list[i]["content"]["src"] = os.path.join("img", img_path + "_" + ".".join([md5, fomt]))
#
#         rdata["msg_list"] = msg_list
#         rdata["myuserdata"] = rdata["user_list"][rdata["my_wxid"]]
#         rdata["myuserdata"]["chat_count"] = len(rdata["msg_list"])
#         save_data = rdata
#         save_json_path = os.path.join(outpath, "data")
#         if not os.path.exists(save_json_path):
#             os.makedirs(save_json_path)
#         with open(os.path.join(save_json_path, "msg_user.json"), "w", encoding="utf-8") as f:
#             json.dump(save_data, f, ensure_ascii=False)
#
#         json_base64 = gen_base64(os.path.join(save_json_path, "msg_user.json"))
#         html = html.replace('"./data/msg_user.json"', f'"{json_base64}"')
#
#         with open(os.path.join(outpath, "index.html"), 'w', encoding='utf-8') as f:
#             f.write(html)
#         return ReJson(0, outpath)
#
#     elif export_type == "pdf":
#         pass
#     elif export_type == "docx":
#         pass
#     else:
#         return ReJson(1002)
#
#     return ReJson(9999, "")


# end 导出聊天记录 *******************************************************************************************************

# start 聊天记录分析api **************************************************************************************************

@api.route('/api/date_count', methods=["GET", 'POST'])
@error9999
def get_date_count():
    """
    获取日期统计
    """
    my_wxid = read_session(g.sf, "test", "last")
    if not my_wxid: return ReJson(1001, body="my_wxid is required")
    merge_path = read_session(g.sf, my_wxid, "merge_path")
    date_count = ParsingMSG(merge_path).date_count()
    return ReJson(0, date_count)


@api.route('/api/wordcloud', methods=["GET", 'POST'])
@error9999
def wordcloud():
    pass


# start 这部分为专业工具的api *********************************************************************************************

@api.route('/api/wxinfo', methods=["GET", 'POST'])
@error9999
def get_wxinfo():
    """
    获取微信信息
    :return:
    """
    import pythoncom
    pythoncom.CoInitialize()
    wxinfos = read_info(VERSION_LIST)
    pythoncom.CoUninitialize()
    return ReJson(0, wxinfos)


@api.route('/api/biasaddr', methods=["GET", 'POST'])
@error9999
def biasaddr():
    """
    BiasAddr
    :return:
    """
    mobile = request.json.get("mobile")
    name = request.json.get("name")
    account = request.json.get("account")
    key = request.json.get("key", "")
    wxdbPath = request.json.get("wxdbPath", "")
    if not mobile or not name or not account:
        return ReJson(1002)
    pythoncom.CoInitialize()
    rdata = BiasAddr(account, mobile, name, key, wxdbPath).run()
    return ReJson(0, str(rdata))


@api.route('/api/decrypt', methods=["GET", 'POST'])
@error9999
def decrypt():
    """
    解密
    :return:
    """
    key = request.json.get("key")
    if not key:
        return ReJson(1002)
    wxdb_path = request.json.get("wxdbPath")
    if not wxdb_path:
        return ReJson(1002)
    out_path = request.json.get("outPath")
    if not out_path:
        out_path = g.tmp_path
    wxinfos = batch_decrypt(key, wxdb_path, out_path=out_path)
    return ReJson(0, str(wxinfos))


@api.route('/api/merge', methods=["GET", 'POST'])
@error9999
def merge():
    """
    合并
    :return:
    """
    wxdb_path = request.json.get("dbPath")
    if not wxdb_path:
        return ReJson(1002)
    out_path = request.json.get("outPath")
    if not out_path:
        return ReJson(1002)
    rdata = merge_db(wxdb_path, out_path)
    return ReJson(0, str(rdata))


# END 这部分为专业工具的api ***********************************************************************************************

# 关于、帮助、设置 *******************************************************************************************************
@api.route('/api/check_update', methods=["GET", 'POST'])
@error9999
def check_update():
    """
    检查更新
    :return:
    """
    url = "https://api.github.com/repos/xaoyaoo/PyWxDump/tags"
    try:
        import requests
        res = requests.get(url)
        if res.status_code == 200:
            data = res.json()
            NEW_VERSION = data[0].get("name")
            if NEW_VERSION[1:] != pywxdump.__version__:
                msg = "有新版本"
            else:
                msg = "已经是最新版本"
            return ReJson(0, body={"msg": msg, "latest_version": NEW_VERSION,
                                   "latest_url": "https://github.com/xaoyaoo/PyWxDump/releases/tag/" + NEW_VERSION})
        else:
            return ReJson(2001, body="status_code is not 200")
    except Exception as e:
        return ReJson(9999, msg=str(e))


@api.route('/api/version', methods=["GET", 'POST'])
@error9999
def version():
    """
    版本
    :return:
    """
    return ReJson(0, pywxdump.__version__)


# END 关于、帮助、设置 ***************************************************************************************************


@api.route('/')
@error9999
def index():
    return render_template('index.html')



# -*- coding: utf-8 -*-#
# -------------------------------------------------------------------------------
# Name:         exportCSV.py
# Description:  
# Author:       xaoyaoo
# Date:         2024/04/20
# -------------------------------------------------------------------------------
import csv
import json
import os
from ..parsingMSG import ParsingMSG


def export_csv(wxid, outpath, msg_path, page_size=5000):
    if not os.path.exists(outpath):
        outpath = os.path.join(os.getcwd(), "export" + os.sep + wxid)
        if not os.path.exists(outpath):
            os.makedirs(outpath)

    pmsg = ParsingMSG(msg_path)

    count = pmsg.msg_count(wxid)
    chatCount = count.get(wxid, 0)
    if chatCount == 0:
        return False, "没有聊天记录"

    if page_size > chatCount:
        page_size = chatCount + 1

    for i in range(0, chatCount, page_size):
        start_index = i
        data, wxid_list = pmsg.msg_list(wxid, start_index, page_size)

        if len(data) == 0:
            return False, "没有聊天记录"

        save_path = os.path.join(outpath, f"{wxid}_{i}_{i + page_size}.csv")

        with open(save_path, "w", encoding="utf-8", newline='') as f:
            csv_writer = csv.writer(f, quoting=csv.QUOTE_MINIMAL)

            csv_writer.writerow(["id", "MsgSvrID", "type_name", "is_sender", "talker", "room_name", "content",
                                 "CreateTime"])
            for row in data:
                id = row.get("id", "")
                MsgSvrID = row.get("MsgSvrID", "")
                type_name = row.get("type_name", "")
                is_sender = row.get("is_sender", "")
                talker = row.get("talker", "")
                room_name = row.get("room_name", "")
                content = row.get("content", "")
                CreateTime = row.get("CreateTime", "")

                content = json.dumps(content, ensure_ascii=False)
                csv_writer.writerow([id, MsgSvrID, type_name, is_sender, talker, room_name, content, CreateTime])

    return True, f"导出成功: {outpath}"


if __name__ == '__main__':
    pass



# -*- coding: utf-8 -*-#
# -------------------------------------------------------------------------------
# Name:         __init__.py.py
# Description:  
# Author:       xaoyaoo
# Date:         2023/10/14
# -------------------------------------------------------------------------------
# from .analyzer.db_parsing import read_img_dat, read_emoji, decompress_CompressContent, read_audio_buf, read_audio, \
#     parse_xml_string, read_BytesExtra
# from .ui import app_show_chat, get_user_list, export
from .wx_info import BiasAddr, read_info, get_wechat_db, batch_decrypt, decrypt, get_core_db
from .wx_info import merge_copy_db, merge_msg_db, merge_media_msg_db, merge_db, decrypt_merge, merge_real_time_db, \
    all_merge_real_time_db
from .analyzer import DBPool
from .dbpreprocess import get_user_list, get_recent_user_list, wxid2userinfo, ParsingMSG, ParsingMicroMsg, \
    ParsingMediaMSG, ParsingOpenIMContact, ParsingFavorite
from .server import start_falsk
import os, json

try:
    VERSION_LIST_PATH = os.path.join(os.path.dirname(__file__), "version_list.json")
    with open(VERSION_LIST_PATH, "r", encoding="utf-8") as f:
        VERSION_LIST = json.load(f)
except:
    VERSION_LIST = {}
    VERSION_LIST_PATH = None

# PYWXDUMP_ROOT_PATH = os.path.dirname(__file__)
# db_init = DBPool("DBPOOL_INIT")

__version__ = "3.0.36"



# -*- coding: utf-8 -*-#
# -------------------------------------------------------------------------------
# Name:         __init__.py.py
# Description:  
# Author:       xaoyaoo
# Date:         2024/04/15
# -------------------------------------------------------------------------------
import pandas as pd

from .parsingFavorite import ParsingFavorite
from .parsingMSG import ParsingMSG
from .parsingMicroMsg import ParsingMicroMsg
from .parsingMediaMSG import ParsingMediaMSG
from .parsingOpenIMContact import ParsingOpenIMContact
from .utils import download_file

from .export.exportCSV import export_csv
from .export.exportJSON import export_json


def get_user_list(MicroMsg_db_path, OpenIMContact_db_path=None, word=None):
    """
    获取联系人列表
    :param MicroMsg_db_path: MicroMsg.db 文件路径
    :param OpenIMContact_db_path: OpenIMContact.db 文件路径
    :param word: 搜索关键字
    :return: 联系人列表
    """
    # 连接 MicroMsg.db 数据库，并执行查询
    if not MicroMsg_db_path:
        return []
    parsing_micromsg = ParsingMicroMsg(MicroMsg_db_path)
    users = parsing_micromsg.user_list(word=word)
    # 如果有 OpenIMContact.db 文件，获取 OpenIMContact.db 中的联系人信息
    if OpenIMContact_db_path:
        parsing_openimcontact = ParsingOpenIMContact(OpenIMContact_db_path)
        users += parsing_openimcontact.user_list(word=word)
    # 去重
    # print(users)
    unique_users = [dict(t) for t in {tuple(d.items()) for d in users}]
    return unique_users


def get_recent_user_list(MicroMsg_db_path, OpenIMContact_db_path=None, limit=200):
    """
    获取联系人列表
    :param MicroMsg_db_path: MicroMsg.db 文件路径
    :param OpenIMContact_db_path: OpenIMContact.db 文件路径
    :param limit: 最大数量
    :return: 联系人列表
    """
    # 连接 MicroMsg.db 数据库，并执行查询
    if not MicroMsg_db_path:
        return []
    parsing_micromsg = ParsingMicroMsg(MicroMsg_db_path)
    recent_users = parsing_micromsg.recent_chat_wxid()  # [{"wxid": username, "LastReadedCreateTime": LastReadedCreateTime, "LastReadedSvrId": LastReadedSvrId},]
    recent_users = pd.DataFrame(recent_users, columns=["wxid", "LastReadedCreateTime", "LastReadedSvrId"])
    recent_users = recent_users.sort_values(by="LastReadedCreateTime", ascending=False)
    recent_users = recent_users.drop_duplicates(subset=["wxid"], keep="first").head(limit)

    users = get_user_list(MicroMsg_db_path, OpenIMContact_db_path)
    users = pd.DataFrame(users)

    users = pd.merge(users, recent_users, on="wxid", how="right")
    # users = users.drop_duplicates(subset=["wxid"], keep="last")  # 保留最新的
    users = users.sort_values(by="LastReadedCreateTime", ascending=False) if not users.empty else users
    users = users.drop_duplicates(subset=["wxid"], keep="first")  # 保留最新的
    users = users.fillna("")
    users = users.to_dict(orient="records")
    return users


def wxid2userinfo(MicroMsg_db_path, OpenIMContact_db_path, wxid):
    """
    获取联系人信息
    :param MicroMsg_db_path: MicroMsg.db 文件路径
    :param OpenIMContact_db_path: OpenIMContact.db 文件路径
    :param wxid: 微信id,可以是单个id，也可以是多个id，使用list传入
    :return: 联系人信息 {wxid: {wxid: wxid, nickname: nickname, remark: remark, account: account, describe: describe, headImgUrl: headImgUrl}}
    """
    # 连接 MicroMsg.db 数据库，并执行查询
    parsing_micromsg = ParsingMicroMsg(MicroMsg_db_path)
    users = parsing_micromsg.wxid2userinfo(wxid)
    # {'wxid_uw8ruinee7zq12': {'wxid': 'wxid_uw8ruinee7zq12', 'nickname': '2021年', 'remark': '于浩', 'account': 'yh13327404424', 'describe': '', 'headImgUrl': 'https://wx.qlogo.cn/mmhead/ver_1/LLibM2qUys7nBt9Hl8uuTQkn9ILFicoImlt2616ZNGoIvRbA8VmJ0Vibhd3V96JFfxQ25Tj1nRWTsXYDdH3z2FAQkQDXSnjS5PBuSraey4ZnoooOkEu2e3DjXbJaJJXKUib1/0'}}
    # 如果有 OpenIMContact.db 文件，获取 OpenIMContact.db 中的联系人信息
    if OpenIMContact_db_path:
        parsing_openimcontact = ParsingOpenIMContact(OpenIMContact_db_path)
        users.update(parsing_openimcontact.wxid2userinfo(wxid))
    return users



# -*- coding: utf-8 -*-#
# -------------------------------------------------------------------------------
# Name:         dbbase.py
# Description:  
# Author:       xaoyaoo
# Date:         2024/04/15
# -------------------------------------------------------------------------------
import os
import sqlite3
import logging


class DatabaseBase:
    _singleton_instances = {}  # 使用字典存储不同db_path对应的单例实例
    _connection_pool = {}  # 使用字典存储不同db_path对应的连接池
    _class_name = "DatabaseBase"

    def __new__(cls, db_path):
        if cls._class_name not in cls._singleton_instances:
            cls._singleton_instances[cls._class_name] = super().__new__(cls)
        return cls._singleton_instances[cls._class_name]

    def __init__(self, db_path):
        self._db_path = db_path
        self._db_connection = self._connect_to_database(db_path)

    @classmethod
    def _connect_to_database(cls, db_path):
        if not os.path.exists(db_path):
            raise FileNotFoundError(f"文件不存在: {db_path}")
        if db_path in cls._connection_pool and cls._connection_pool[db_path] is not None:
            return cls._connection_pool[db_path]
        connection = sqlite3.connect(db_path, check_same_thread=False)
        logging.info(f"{connection} 连接句柄创建 {db_path}")
        return connection

    def execute_sql(self, sql, params=None):
        """
        执行SQL语句
        :param sql: SQL语句 (str)
        :param params: 参数 (tuple)
        :return: 查询结果 (list)
        """
        # 检测数据库连接是否关闭
        if not self._db_connection:
            logging.warning(f"重新连接数据库 - {self._db_path}")
            self._connect_to_database(self._db_path)
        connection = self._db_connection
        try:
            # connection.text_factory = bytes
            cursor = connection.cursor()
            if params:
                cursor.execute(sql, params)
            else:
                cursor.execute(sql)
            return cursor.fetchall()
        except Exception as e1:
            try:
                connection.text_factory = bytes
                cursor = connection.cursor()
                if params:
                    cursor.execute(sql, params)
                else:
                    cursor.execute(sql)
                rdata = cursor.fetchall()
                connection.text_factory = str
                return rdata
            except Exception as e2:
                logging.error(f"**********\nSQL: {sql}\nparams: {params}\n{e1}\n{e2}\n**********")
                return None

    def close_connection(self):
        if self._db_connection:
            self._db_connection.close()
            logging.info(f"关闭数据库 - {self._db_path}")
            self._db_connection = None

    def close_all_connection(self):
        for db_path in self._connection_pool:
            if self._connection_pool[db_path]:
                self._connection_pool[db_path].close()
                logging.info(f"关闭数据库 - {db_path}")
                self._connection_pool[db_path] = None

    def show__singleton_instances(self):
        print(self._singleton_instances)

    def __del__(self):
        self.close_connection()
        # del self._singleton_instances[self._db_path]


if __name__ == '__main__':
    a = DatabaseBase("test.db")
    b = DatabaseBase("test1.db")

    d1 = a.execute_sql("select * from sqlite_master;")
    d2 = b.execute_sql("select * from sqlite_master;")
    print([i[1] for i in d1])
    print([i[1] for i in d2])

    a.close_connection()
    b.close_connection()



# -*- coding: utf-8 -*-#
# -------------------------------------------------------------------------------
# Name:         parsingMicroMsg.py
# Description:  
# Author:       xaoyaoo
# Date:         2024/04/15
# -------------------------------------------------------------------------------
import logging

from .dbbase import DatabaseBase
from .utils import timestamp2str, bytes2str

import blackboxprotobuf


class ParsingMicroMsg(DatabaseBase):
    _class_name = "MicroMsg"

    def __init__(self, db_path):
        super().__init__(db_path)

    def get_BytesExtra(self, BytesExtra):
        if BytesExtra is None or not isinstance(BytesExtra, bytes):
            return None
        try:
            deserialize_data, message_type = blackboxprotobuf.decode_message(BytesExtra)
            return deserialize_data
        except Exception as e:
            return None

    def get_ExtraBuf(self, ExtraBuf: bytes):
        """
        读取ExtraBuf（联系人表）
        :param ExtraBuf:
        :return:
        """
        if not ExtraBuf:
            return None
        try:
            buf_dict = {
                'DDF32683': '0', '74752C06': '性别[1男2女]', '88E28FCE': '2', '761A1D2D': '3', '0263A0CB': '4',
                '0451FF12': '5',
                '228C66A8': '6', '46CF10C4': '个性签名', 'A4D9024A': '国', 'E2EAA8D1': '省', '1D025BBF': '市',
                '4D6C4570': '11',
                'F917BCC0': '公司名称', '759378AD': '手机号', '4335DFDD': '14', 'DE4CDAEB': '15', 'A72BC20A': '16',
                '069FED52': '17',
                '9B0F4299': '18', '3D641E22': '19', '1249822C': '20', '4EB96D85': '企微属性', 'B4F73ACB': '22',
                '0959EB92': '23',
                '3CF4A315': '24', 'C9477AC60201E44CD0E8': '26', 'B7ACF0F5': '28', '57A7B5A8': '29',
                '81AE19B4': '朋友圈背景',
                '695F3170': '31', 'FB083DD9': '32', '0240E37F': '33', '315D02A3': '34', '7DEC0BC3': '35',
                '0E719F13': '备注图片',
                '16791C90': '37'
            }

            rdata = {}
            for buf_name in buf_dict:
                rdata_name = buf_dict[buf_name]
                buf_name = bytes.fromhex(buf_name)
                offset = ExtraBuf.find(buf_name)
                if offset == -1:
                    rdata[rdata_name] = ""
                    continue
                offset += len(buf_name)
                type_id = ExtraBuf[offset: offset + 1]
                offset += 1

                if type_id == b"\x04":
                    rdata[rdata_name] = int.from_bytes(ExtraBuf[offset: offset + 4], "little")

                elif type_id == b"\x18":
                    length = int.from_bytes(ExtraBuf[offset: offset + 4], "little")
                    rdata[rdata_name] = ExtraBuf[offset + 4: offset + 4 + length].decode("utf-16").rstrip("\x00")

                elif type_id == b"\x17":
                    length = int.from_bytes(ExtraBuf[offset: offset + 4], "little")
                    rdata[rdata_name] = ExtraBuf[offset + 4: offset + 4 + length].decode("utf-8").rstrip("\x00")

                elif type_id == b"\x05":
                    rdata[rdata_name] = f"0x{ExtraBuf[offset: offset + 8].hex()}"
            return rdata
        except Exception as e:
            print(f'解析错误:\n{e}')
            return None

    def ChatRoom_RoomData(self, RoomData):
        # 读取群聊数据,主要为 wxid，以及对应昵称
        if RoomData is None or not isinstance(RoomData, bytes):
            return None
        try:
            data = self.get_BytesExtra(RoomData)
            bytes2str(data)
            return data
        except Exception as e:
            return None

    def wxid2userinfo(self, wxid):
        """
        获取单个联系人信息
        :param wxid: 微信id,可以是单个id，也可以是id列表
        :return: 联系人信息
        """
        if isinstance(wxid, str):
            wxid = [wxid]
        elif isinstance(wxid, list):
            wxid = wxid
        else:
            return {}
        wxid = "','".join(wxid)
        wxid = f"'{wxid}'"
        # 获取username是wx_id的用户
        sql = ("SELECT A.UserName, A.NickName, A.Remark,A.Alias,A.Reserved6,B.bigHeadImgUrl,A.LabelIDList "
               "FROM Contact A,ContactHeadImgUrl B "
               f"WHERE A.UserName = B.usrName AND A.UserName in ({wxid}) "
               "ORDER BY NickName ASC;")
        result = self.execute_sql(sql)
        if not result:
            return {}
        users = {}
        for row in result:
            # 获取wxid,昵称，备注，描述，头像
            username, nickname, remark, Alias, describe, headImgUrl, LabelIDList = row
            LabelIDList = LabelIDList.split(",") if LabelIDList else []
            users[username] = {"wxid": username, "nickname": nickname, "remark": remark, "account": Alias,
                               "describe": describe, "headImgUrl": headImgUrl, "LabelIDList": tuple(LabelIDList)}
        return users

    def user_list(self, word=None):
        """
        获取联系人列表
        :param word 查询关键字，可以是用户名、昵称、备注、描述，允许拼音
        :return: 联系人列表
        """
        users = []
        sql = (
            "SELECT A.UserName, A.NickName, A.Remark,A.Alias,A.Reserved6,B.bigHeadImgUrl,A.LabelIDList "
            "FROM Contact A left join ContactHeadImgUrl B on A.UserName==B.usrName "
            "ORDER BY A.NickName DESC;")
        if word:
            sql = sql.replace("ORDER BY A.NickName DESC;",
                              f"where "
                              f"A.UserName LIKE '%{word}%' "
                              f"OR A.NickName LIKE '%{word}%' "
                              f"OR A.Remark LIKE '%{word}%' "
                              f"OR A.Alias LIKE '%{word}%' "
                              f"OR A.QuanPin LIKE LOWER('%{word}%') "
                              f"OR LOWER(A.PYInitial) LIKE LOWER('%{word}%') "
                              # f"OR A.Reserved6 LIKE '%{word}%' "
                              "ORDER BY A.NickName DESC;")
        result = self.execute_sql(sql)
        if not result:
            return []
        for row in result:
            # 获取wxid,昵称，备注，描述，头像,标签
            username, nickname, remark, Alias, describe, headImgUrl, LabelIDList = row
            LabelIDList = LabelIDList.split(",") if LabelIDList else []
            users.append(
                {"wxid": username, "nickname": nickname, "remark": remark, "account": Alias,
                 "describe": describe, "headImgUrl": headImgUrl if headImgUrl else "",
                 "LabelIDList": tuple(LabelIDList)})
        return users

    def user_list_by_label(self, label_id):
        """
        获取标签联系人列表
        :param label_id: 标签id
        :return: 标签联系人列表
        """
        users = []
        sql = (
            "SELECT A.UserName, A.NickName, A.Remark,A.Alias,A.Reserved6,B.bigHeadImgUrl,A.LabelIDList "
            "FROM Contact A left join ContactHeadImgUrl B on A.UserName==B.usrName "
            f"where A.LabelIDList LIKE '%{label_id}%' "
            "ORDER BY A.NickName DESC;")
        result = self.execute_sql(sql)
        if not result:
            return []
        for row in result:
            # 获取wxid,昵称，备注，描述，头像,标签
            username, nickname, remark, Alias, describe, headImgUrl, LabelIDList = row
            LabelIDList = LabelIDList.split(",") if LabelIDList else []
            users.append(
                {"wxid": username, "nickname": nickname, "remark": remark, "account": Alias,
                 "describe": describe, "headImgUrl": headImgUrl if headImgUrl else "",
                 "LabelIDList": tuple(LabelIDList)})
        return users

    def recent_chat_wxid(self):
        """
        获取最近聊天的联系人
        :return: 最近聊天的联系人
        """
        users = []
        sql = (
            "SELECT C.Username, C.LastReadedCreateTime,C.LastReadedSvrId "
            "FROM ChatInfo C "
            "ORDER BY C.LastReadedCreateTime DESC;")
        result = self.execute_sql(sql)
        if not result:
            return []
        for row in result:
            # 获取用户名、昵称、备注和聊天记录数量
            username, LastReadedCreateTime, LastReadedSvrId = row
            LastReadedCreateTime = timestamp2str(LastReadedCreateTime) if LastReadedCreateTime else None
            users.append(
                {"wxid": username, "LastReadedCreateTime": LastReadedCreateTime, "LastReadedSvrId": LastReadedSvrId})
        return users

    def chatroom_list(self, roomwxid=None):
        """
        获取群聊列表
        :param MicroMsg_db_path: MicroMsg.db 文件路径
        :return: 群聊列表
        """
        rooms = []
        # 连接 MicroMsg.db 数据库，并执行查询
        sql = (
            "SELECT A.ChatRoomName,A.UserNameList, A.DisplayNameList,A.RoomData, B.Announcement,B.AnnouncementEditor "
            "FROM ChatRoom A,ChatRoomInfo B "
            "where A.ChatRoomName==B.ChatRoomName "
            "ORDER BY A.ChatRoomName ASC;")
        if roomwxid:
            sql = sql.replace("ORDER BY A.ChatRoomName ASC;",
                              f"and A.ChatRoomName LIKE '%{roomwxid}%' "
                              "ORDER BY A.ChatRoomName ASC;")
        result = self.execute_sql(sql)
        if not result:
            return []
        room_datas = []
        for row in result:
            # 获取用户名、昵称、备注和聊天记录数量
            ChatRoomName, UserNameList, DisplayNameList, RoomData, Announcement, AnnouncementEditor = row
            UserNameList = UserNameList.split("^G")
            DisplayNameList = DisplayNameList.split("^G")
            RoomData = self.ChatRoom_RoomData(RoomData)
            wxid2remark = {}
            if RoomData:
                rd = []
                for k, v in RoomData.items():
                    if isinstance(v, list):
                        rd += v
                for i in rd:
                    try:
                        if isinstance(i, dict) and isinstance(i.get('1'), str) and i.get('2'):
                            wxid2remark[i['1']] = i["2"]
                    except Exception as e:
                        logging.error(f"wxid2remark: ChatRoomName:{ChatRoomName}, {i} error:{e}")
            rooms.append(
                {"ChatRoomName": ChatRoomName, "UserNameList": UserNameList, "DisplayNameList": DisplayNameList,
                 "Announcement": Announcement, "AnnouncementEditor": AnnouncementEditor, "wxid2remark": wxid2remark})
        return rooms

    def labels_dict(self, id_is_key=True):
        """
        读取标签列表
        :param label_list:
        :return:
        """
        sql = "SELECT LabelId, LabelName FROM ContactLabel ORDER BY LabelName ASC;"
        result = self.execute_sql(sql)
        if not result:
            return []
        if id_is_key:
            labels = {row[0]: row[1] for row in result}
        else:
            labels = {row[1]: row[0] for row in result}
        return labels



# -*- coding: utf-8 -*-#
# -------------------------------------------------------------------------------
# Name:         MediaMSG_parsing.py
# Description:  
# Author:       xaoyaoo
# Date:         2024/04/15
# -------------------------------------------------------------------------------
from .dbbase import DatabaseBase
from .utils import silk2audio


class ParsingMediaMSG(DatabaseBase):
    _class_name = "MediaMSG"
    def __init__(self, db_path):
        super().__init__(db_path)

    def get_audio(self, MsgSvrID, is_play=False, is_wave=False, save_path=None, rate=24000):
        sql = "select Buf from Media where Reserved0=? "
        DBdata = self.execute_sql(sql, (MsgSvrID,))
        if not DBdata:
            return False
        if len(DBdata) == 0:
            return False
        data = DBdata[0][0]  # [1:] + b'\xFF\xFF'
        try:
            pcm_data = silk2audio(buf_data=data, is_play=is_play, is_wave=is_wave, save_path=save_path, rate=rate)
            return pcm_data
        except Exception as e:
            return False



# -*- coding: utf-8 -*-#
# -------------------------------------------------------------------------------
# Name:         exportCSV.py
# Description:  
# Author:       xaoyaoo
# Date:         2024/04/20
# -------------------------------------------------------------------------------
import json
import os
from ..parsingMSG import ParsingMSG


def export_json(wxid, outpath, msg_path):
    if not os.path.exists(outpath):
        outpath = os.path.join(os.getcwd(), "export" + os.sep + wxid)
        if not os.path.exists(outpath):
            os.makedirs(outpath)

    pmsg = ParsingMSG(msg_path)

    count = pmsg.msg_count(wxid)
    chatCount = count.get(wxid, 0)
    if chatCount == 0:
        return False, "没有聊天记录"

    page_size = chatCount + 1
    for i in range(0, chatCount, page_size):
        start_index = i
        data, wxid_list = pmsg.msg_list(wxid, start_index, page_size)
        if len(data) == 0:
            return False, "没有聊天记录"
        save_path = os.path.join(outpath, f"{wxid}_{i}_{i + page_size}.json")
        with open(save_path, "w", encoding="utf-8") as f:
            json.dump(data, f, ensure_ascii=False, indent=4)
    return True, f"导出成功: {outpath}"


if __name__ == '__main__':
    pass



# -*- coding: utf-8 -*-#
# -------------------------------------------------------------------------------
# Name:         server.py
# Description:  
# Author:       xaoyaoo
# Date:         2024/01/04
# -------------------------------------------------------------------------------
import os
import subprocess
import sys
import time


def start_falsk(merge_path="", wx_path="", key="", my_wxid="", port=5000, online=False, debug=False,
                isopenBrowser=True):
    """
    启动flask
    :param merge_path:  合并后的数据库路径
    :param wx_path:  微信文件夹的路径（用于显示图片）
    :param key:  密钥
    :param my_wxid:  微信账号(本人微信id)
    :param port:  端口号
    :param online:  是否在线查看(局域网查看)
    :param debug:  是否开启debug模式
    :param isopenBrowser:  是否自动打开浏览器
    :return:
    """
    tmp_path = os.path.join(os.getcwd(), "wxdump_tmp")  # 临时文件夹,用于存放图片等
    if not os.path.exists(tmp_path):
        os.makedirs(tmp_path)
        print(f"[+] 创建临时文件夹：{tmp_path}")

    session_file = os.path.join(tmp_path, "conf.json")  # 用于存放各种基础信息

    from flask import Flask, g
    from flask_cors import CORS
    from pywxdump.api import api, read_session, save_session
    import logging

    # 检查端口是否被占用
    if online:
        host = '0.0.0.0'
    else:
        host = "127.0.0.1"

    app = Flask(__name__, template_folder='./ui/web', static_folder='./ui/web/assets/', static_url_path='/assets/')

    # 设置超时时间为 1000 秒
    app.config['TIMEOUT'] = 1000
    app.secret_key = 'secret_key'

    app.logger.setLevel(logging.ERROR)

    CORS(app, resources={r"/*": {"origins": "*"}}, supports_credentials=True)  # 允许所有域名跨域

    @app.after_request  # 请求后的处理 用于解决部分用户浏览器不支持flask以及vue的js文件返回问题
    def changeHeader(response):
        disposition = response.get_wsgi_headers('environ').get(
            'Content-Disposition') or ''  # 获取返回头文件名描述，如'inline; filename=index.562b9b5a.js'
        if disposition.rfind('.js') == len(disposition) - 3:
            response.mimetype = 'application/javascript'
        return response

    @app.before_request
    def before_request():

        g.tmp_path = tmp_path  # 临时文件夹,用于存放图片等
        g.sf = session_file  # 用于存放各种基础信息

    if merge_path: save_session(session_file, "test", "merge_path", merge_path)
    if wx_path: save_session(session_file, "test", "wx_path", wx_path)
    if key: save_session(session_file, "test", "key", key)
    if my_wxid: save_session(session_file, "test", "my_wxid", my_wxid)
    if not os.path.exists(session_file):
        save_session(session_file, "test", "last", my_wxid)

    app.register_blueprint(api)
    if isopenBrowser:
        try:
            # 自动打开浏览器
            url = f"http://127.0.0.1:{port}/"
            # 根据操作系统使用不同的命令打开默认浏览器
            if sys.platform.startswith('darwin'):  # macOS
                subprocess.call(['open', url])
            elif sys.platform.startswith('win'):  # Windows
                subprocess.call(['start', url], shell=True)
            elif sys.platform.startswith('linux'):  # Linux
                subprocess.call(['xdg-open', url])
            else:
                print("Unsupported platform, can't open browser automatically.")
        except Exception as e:
            pass

    def is_port_in_use(host, port):
        import socket
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
            try:
                s.bind((host, port))
            except socket.error:
                return True
        return False

    if is_port_in_use(host, port):
        print(f"Port {port} is already in use. Choose a different port.")
        input("Press Enter to exit...")
    else:
        time.sleep(1)
        print("[+] 请使用浏览器访问 http://127.0.0.1:5000/ 查看聊天记录")
        app.run(host=host, port=port, debug=debug)


if __name__ == '__main__':
    merge_path = r"****.db"

    wx_path = r"****"
    my_wxid = "****"

    start_falsk(merge_path=merge_path, wx_path=wx_path, my_wxid=my_wxid,
                port=5000, online=False, debug=False, isopenBrowser=False)



# -*- coding: utf-8 -*-#
# -------------------------------------------------------------------------------
# Name:         parsingFavorite.py
# Description:  
# Author:       xaoyaoo
# Date:         2024/05/18
# -------------------------------------------------------------------------------
import pandas as pd

from .dbbase import DatabaseBase
from .utils import timestamp2str, xml2dict


# * FavItems：收藏的消息条目列表
# * FavDataItem：收藏的具体数据。大概可以确定以下两点
#     * 即使只是简单收藏一篇公众号文章也会在 FavDataItem 中有一个对应的记录
#     * 对于收藏的合并转发类型的消息，合并转发中的每一条消息在 FavDataItem 中都是一个独立的记录
# * FavTags：为收藏内容添加的标签


def FavoriteTypeId2Name(Type):
    TypeNameDict = {
        1: "文本",  # 文本 已测试
        2: "图片",  # 图片 已测试
        3: "语音",  # 语音
        4: "视频",  # 视频 已测试
        5: "链接",  # 链接 已测试
        6: "位置",  # 位置
        7: "小程序",  # 小程序
        8: "文件",  # 文件 已测试
        14: "聊天记录",  # 聊天记录 已测试
        16: "群聊视频",  # 群聊中的视频 可能
        18: "笔记"  # 笔记 已测试
    }
    return TypeNameDict.get(Type, "未知")


class ParsingFavorite(DatabaseBase):
    _class_name = "Favorite"

    def __init__(self, db_path):
        super().__init__(db_path)

    def get_tags(self, LocalID):
        """
        return: {LocalID: TagName}
        """
        if LocalID is None:
            sql = "select LocalID, TagName from FavTagDatas order by ServerSeq"
        else:
            sql = "select LocalID, TagName from FavTagDatas where LocalID = '%s' order by ServerSeq " % LocalID
        tags = self.execute_sql(sql)  # [(1, 797940830, '程序语言类'), (2, 806153863, '账单')]
        # 转换为字典
        tags = {tag[0]: tag[1] for tag in tags}
        return tags

    def get_FavBindTags(self):
        """
        return: [(FavLocalID, TagName)]
        """
        sql = "select A.FavLocalID, B.TagName from FavBindTagDatas A, FavTagDatas B where A.TagLocalID = B.LocalID"
        FavBindTags = self.execute_sql(sql)
        return FavBindTags

    def get_favorite(self):
        """
        return: [{FavItemsFields}, {FavItemsFields}]
        """
        FavItemsFields = {
            "FavLocalID": "本地收藏ID",
            "SvrFavId": "服务器收藏ID",
            "SourceId": "源ID",
            "Type": "类型",
            "SourceType": "源类型",
            "LocalStatus": "本地状态",
            "Flag": "标记",
            "Status": "状态",
            "FromUser": "源用户",
            "RealChatName": "实际聊天名称",
            "SearchKey": "搜索关键字",
            "UpdateTime": "更新时间",
            "reseverd0": "预留字段0",
            "XmlBuf": "XML缓冲区"
        }
        FavDataItemFields = {
            "FavLocalID": "本地收藏ID",
            "Type": "类型",
            "DataId": "数据ID",
            "HtmlId": "HTML ID",
            "Datasourceid": "数据源ID",
            "Datastatus": "数据状态",
            "Datafmt": "数据格式",
            "Datatitle": "数据标题",
            "Datadesc": "数据描述",
            "Thumbfullmd5": "缩略图全MD5",
            "Thumbhead256md5": "缩略图头256MD5",
            "Thumbfullsize": "缩略图全尺寸",
            "fullmd5": "全MD5",
            "head256md5": "头256MD5",
            "fullsize": "全尺寸",
            "cdn_thumburl": "CDN缩略图URL",
            "cdn_thumbkey": "CDN缩略图KEY",
            "thumb_width": "缩略图宽度",
            "thumb_height": "缩略图高度",
            "cdn_dataurl": "CDN数据URL",
            "cdn_datakey": "CDN数据KEY",
            "cdn_encryver": "CDN加密版本",
            "duration": "时长",
            "stream_weburl": "流媒体WEB URL",
            "stream_dataurl": "流媒体数据URL",
            "stream_lowbandurl": "流媒体低带宽URL",
            "sourcethumbpath": "源缩略图路径",
            "sourcedatapath": "源数据路径",
            "stream_videoid": "流媒体视频ID",
            "Rerserved1": "保留字段1",
            "Rerserved2": "保留字段2",
            "Rerserved3": "保留字段3",
            "Rerserved4": "保留字段4",
            "Rerserved5": "保留字段5",
            "Rerserved6": "保留字段6",
            "Rerserved7": "保留字段7"
        }

        sql1 = "select " + ",".join(FavItemsFields.keys()) + " from FavItems order by UpdateTime desc"
        sql2 = "select " + ",".join(FavDataItemFields.keys()) + " from FavDataItem B order by B.RecId asc"

        FavItemsList = self.execute_sql(sql1)
        FavDataItemList = self.execute_sql(sql2)
        if FavItemsList is None or len(FavItemsList) == 0:
            return False

        FavDataDict = {}
        if FavDataItemList and len(FavDataItemList) >= 0:
            for item in FavDataItemList:
                data_dict = {}
                for i, key in enumerate(FavDataItemFields.keys()):
                    data_dict[key] = item[i]
                FavDataDict[item[0]] = FavDataDict.get(item[0], []) + [data_dict]
        # 获取标签
        FavTags = self.get_FavBindTags()
        FavTagsDict = {}
        for FavLocalID, TagName in FavTags:
            FavTagsDict[FavLocalID] = FavTagsDict.get(FavLocalID, []) + [TagName]

        pf = pd.DataFrame(FavItemsList)
        pf.columns = FavItemsFields.keys()  # set column names
        pf["UpdateTime"] = pf["UpdateTime"].apply(timestamp2str)  # 处理时间
        pf["XmlBuf"] = pf["XmlBuf"].apply(xml2dict)  # 处理xml
        pf["TypeName"] = pf["Type"].apply(FavoriteTypeId2Name)  # 添加类型名称列
        pf["FavData"] = pf["FavLocalID"].apply(lambda x: FavDataDict.get(x, []))  # 添加数据列
        pf["Tags"] = pf["FavLocalID"].apply(lambda x: FavTagsDict.get(x, []))  # 添加标签列
        pf = pf.fillna("")  # 去掉Nan
        rdata = pf.to_dict(orient="records")
        return rdata



# -*- coding: utf-8 -*-#
# -------------------------------------------------------------------------------
# Name:         utils.py
# Description:  
# Author:       xaoyaoo
# Date:         2023/12/03
# -------------------------------------------------------------------------------
import hashlib
import os
import re
import sqlite3
import time


def time_int2str(time_int):
    """
    时间戳转换为时间字符串
    :param time_int: 时间戳
    :return: 时间字符串
    """
    return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time_int))


def read_dict_all_values(data):
    """
    读取字典中所有的值（单层）
    :param dict_data: 字典
    :return: 所有值的list
    """
    result = []
    if isinstance(data, list):
        for item in data:
            result.extend(read_dict_all_values(item))
    elif isinstance(data, dict):
        for key, value in data.items():
            result.extend(read_dict_all_values(value))
    else:
        if isinstance(data, bytes):
            tmp = data.decode("utf-8")
        else:
            tmp = str(data) if isinstance(data, int) else data
        result.append(tmp)

    for i in range(len(result)):
        if isinstance(result[i], bytes):
            result[i] = result[i].decode("utf-8")
    return result


def match_BytesExtra(BytesExtra, pattern=r"FileStorage(.*?)'"):
    """
    匹配 BytesExtra
    :param BytesExtra: BytesExtra
    :param pattern: 匹配模式
    :return:
    """
    if not BytesExtra:
        return False
    BytesExtra = read_dict_all_values(BytesExtra)
    BytesExtra = "'" + "'".join(BytesExtra) + "'"
    # print(BytesExtra)

    match = re.search(pattern, BytesExtra)
    if match:
        video_path = match.group(0).replace("'", "")
        return video_path
    else:
        return ""


def get_type_name(type_id: tuple):
    """
    获取消息类型名称
    :param type_id: 消息类型ID 元组 eg: (1, 0)
    :return:
    """
    type_name_dict = {
        (1, 0): "文本",
        (3, 0): "图片",
        (34, 0): "语音",
        (43, 0): "视频",
        (47, 0): "动画表情",

        (37, 0): "添加好友",  # 感谢 https://github.com/zhyc9de
        (42, 0): "推荐公众号",  # 感谢 https://github.com/zhyc9de
        (48, 0): "地图信息",  # 感谢 https://github.com/zhyc9de
        (49, 40): "分享收藏夹",  # 感谢  https://github.com/zhyc9de
        (49, 53): "接龙",  # 感谢  https://github.com/zhyc9de

        (49, 0): "文件",
        (49, 1): "类似文字消息而不一样的消息",
        (49, 5): "卡片式链接",
        (49, 6): "文件",
        (49, 8): "用户上传的GIF表情",
        (49, 19): "合并转发的聊天记录",
        (49, 33): "分享的小程序",
        (49, 36): "分享的小程序",
        (49, 57): "带有引用的文本消息",
        (49, 63): "视频号直播或直播回放等",
        (49, 87): "群公告",
        (49, 88): "视频号直播或直播回放等",
        (49, 2000): "转账消息",
        (49, 2003): "赠送红包封面",

        (50, 0): "语音通话",
        (10000, 0): "系统通知",
        (10000, 4): "拍一拍",
        (10000, 8000): "系统通知"
    }

    if type_id in type_name_dict:
        return type_name_dict[type_id]
    else:
        return "未知"


def get_name_typeid(type_name: str):
    """
    获取消息类型名称
    :param type_id: 消息类型ID 元组 eg: (1, 0)
    :return:
    """
    type_name_dict = {
        (1, 0): "文本",
        (3, 0): "图片",
        (34, 0): "语音",
        (43, 0): "视频",
        (47, 0): "动画表情",

        (37, 0): "添加好友",  # 感谢 https://github.com/zhyc9de
        (42, 0): "推荐公众号",  # 感谢 https://github.com/zhyc9de
        (48, 0): "地图信息",  # 感谢 https://github.com/zhyc9de
        (49, 40): "分享收藏夹",  # 感谢  https://github.com/zhyc9de
        (49, 53): "接龙",  # 感谢  https://github.com/zhyc9de

        (49, 0): "文件",
        (49, 1): "类似文字消息而不一样的消息",
        (49, 5): "卡片式链接",
        (49, 6): "文件",
        (49, 8): "用户上传的GIF表情",
        (49, 19): "合并转发的聊天记录",
        (49, 33): "分享的小程序",
        (49, 36): "分享的小程序",
        (49, 57): "带有引用的文本消息",
        (49, 63): "视频号直播或直播回放等",
        (49, 87): "群公告",
        (49, 88): "视频号直播或直播回放等",
        (49, 2000): "转账消息",
        (49, 2003): "赠送红包封面",

        (50, 0): "语音通话",
        (10000, 0): "系统通知",
        (10000, 4): "拍一拍",
        (10000, 8000): "系统通知"
    }
    type_tup = []
    for k, v in type_name_dict.items():
        if v == type_name:
            type_tup.append(k)
    return type_tup


def get_md5(data):
    """
    获取数据的 MD5 值
    :param data: 数据（bytes）
    :return:
    """
    md5 = hashlib.md5()
    md5.update(data)
    return md5.hexdigest()


import threading


def get_thread_id():
    current_thread = threading.current_thread()
    thread_id = current_thread.ident
    return thread_id


class DBPool:
    __db_pool = {}
    __thread_pool = {}

    def __new__(cls, *args, **kwargs):
        if not hasattr(cls, '_instance'):
            cls._instance = super(DBPool, cls).__new__(cls)
        return cls._instance

    @classmethod
    def create_connection(cls, db_path):
        if db_path == "DBPOOL_INIT":
            return
        if not os.path.exists(db_path):
            raise FileNotFoundError(f"数据库文件不存在：{db_path}")

        if db_path not in cls.__db_pool:
            cls.__db_pool[db_path] = sqlite3.connect(db_path, check_same_thread=False)
        cls.connection = cls.__db_pool[db_path]

    def __init__(self, db_path):
        if db_path == "DBPOOL_INIT":
            return
        self.db_path = db_path
        if db_path not in self.__db_pool:
            self.create_connection(db_path)
        self.connection = self.__db_pool.get(db_path)
        # 检查数据库是否关闭
        if not self.connection:
            self.create_connection(db_path)
            self.connection = self.__db_pool.get(db_path)

    def __enter__(self):
        return self.connection

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.connection = None

    def close_all(self):
        for db_path, connection in self.__db_pool.items():
            connection.close()
        self.__db_pool.clear()


def attach_databases(connection, databases):
    """
    将多个数据库附加到给定的SQLite连接。
    参数：
    -连接：SQLite连接
    -数据库：包含数据库别名和文件路径的词典
    """
    cursor = connection.cursor()
    for alias, file_path in databases.items():
        attach_command = f"ATTACH DATABASE '{file_path}' AS {alias};"
        cursor.execute(attach_command)
    connection.commit()


def detach_databases(connection, aliases):
    """
    从给定的 SQLite 连接中分离多个数据库。

    参数：
        - connection： SQLite连接
        - aliases：要分离的数据库别名列表
    """
    cursor = connection.cursor()
    for alias in aliases:
        detach_command = f"DETACH DATABASE {alias};"
        cursor.execute(detach_command)
    connection.commit()


def execute_sql(connection, sql, params=None):
    """
    执行给定的SQL语句，返回结果。
    参数：
        - connection： SQLite连接
        - sql：要执行的SQL语句
        - params：SQL语句中的参数
    """
    try:
        # connection.text_factory = bytes
        cursor = connection.cursor()
        if params:
            cursor.execute(sql, params)
        else:
            cursor.execute(sql)
        return cursor.fetchall()
    except Exception as e1:
        try:
            connection.text_factory = bytes
            cursor = connection.cursor()
            if params:
                cursor.execute(sql, params)
            else:
                cursor.execute(sql)
            rdata = cursor.fetchall()
            connection.text_factory = str
            return rdata
        except Exception as e2:
            print(f"**********\nSQL: {sql}\nparams: {params}\n{e1}\n{e2}\n**********")
            return None


if __name__ == '__main__':
    pass
